htmlScroller = Class.create(
{
	/*
		htmlScroller create a div which scrolls html content into the given container. 
		Note that the container need to have width and height assigned to work properly.
		The new div has the same id as its parent plus '_scroller' (ex: div_container => div_container_scroller).
		
		id_element : id of the html container for the scroller.
		html : content to scroll.
		options :
			- width : of the scroller div, default width of the container.
			- height : of the scroller div, default height of the container.
			- speed : of the scroller, bigger => faster (period = 50ms / speed), default 1.
			- direction : of the scroller can be (top, bottom, right, left), default top.
			- stopOnHover : if true, the scroller stops when the mouse is over, default true.
			- startVisible : if true, the content is visible since the beggin of the scrolling, default false.
			- loop : 
				0 : no loop.
				1 : loop with blank, default option.
				2 : loop without blank, (the content need to have a height/width (top bottom / left right) bigger than the container).
	*/
	initialize : function(id_element, html, options)
	{
		this._scrolling=true;
		this._parent = $(id_element);
		if(typeof options == 'undefined')
			options={width:this._parent.getWidth(), height:this._parent.getHeight(), speed:1, direction:'top', stopOnHover:true, startVisible:false, loop:1};
		this._speed = options.speed || 1 ;
		this._direction = options.direction || 'top' ;
		this._stopOnHover = typeof options.stopOnHover == 'undefined' ? true : options.stopOnHover ;
		this._startVisible = typeof options.startVisible == 'undefined' ? false : options.startVisible ;
		this._loop = typeof options.loop == 'undefined' ? 1 : options.loop ;
		
		this._parent.style.overflow='hidden';
		this._elScroller = Builder.node('DIV');
		this._elScroller.writeAttribute('id', this._parent.id+'_scroller');
		this._elScroller.innerHTML=html;
		
		this._parent.insert({bottom:this._elScroller});
		
		this._width = options.width || this._elScroller.getWidth() ;
		this._height = options.height || this._elScroller.getHeight() ;
		
		if ( (typeof options.onEndScroll) == 'function')
			this._onEndScrollHandler = options.onEndScroll.bind(this);
		else
			this._onEndScrollHandler = function () {};
		
		this._parent.setStyle({position: 'relative'});
		var top =  (!this._startVisible ? (this._direction=='top' ? this._parent.getHeight() : (this._direction=='bottom' ? -this._height : 0 )) : 0)+'px';
		var left =  (!this._startVisible ? (this._direction=='left' ? this._parent.getWidth() : (this._direction=='right' ? -this._width : 0 )) : 0)+'px';
		this._elScroller.setStyle({
			width: this._width+'px', 
			height: this._height+'px', 
			position: 'absolute',
			top: top,
			left: left
		});
		
		if (this._loop == 2)
		{
			this._elScroller2 = this._elScroller.clone() ;
			this._elScroller2.writeAttribute('id', this._parent.id+'_scroller2');
			this._elScroller2.innerHTML=html;
			var top = (parseInt(this._elScroller.style.top)+(this._direction=='top' ? this._height : (this._direction=='bottom' ? -this._height : 0 )))+'px';
			var left = (parseInt(this._elScroller.style.left)+(this._direction=='left' ? this._width : (this._direction=='right' ? -this._width : 0 )))+'px';
			this._elScroller2.setStyle({top: top, left: left});
			this._parent.insert({bottom:this._elScroller2});
		}
		
		this._executer = new PeriodicalExecuter(function(){this.scroll();}.bind(this), 1/(20*this._speed));
		
		if(this._stopOnHover)
		{
			Event.observe(this._parent.id, 'mouseenter', function(){
				this._scrolling = false ;
			}.bind(this));
			Event.observe(this._parent.id, 'mouseleave', function(){
				this._scrolling = true ;
			}.bind(this));
		}
	},
	_stopScrolling: function()
	{
		this._executer.stop();
		this._onEndScrollHandler();
	},
	scroll:function() // Function called to scroll
	{
		if(!this._scrolling)
				return;
		var top = parseInt(this._elScroller.style.top) ;
		var left = parseInt(this._elScroller.style.left) ;
		if(this._direction == 'top')
		{
			top+= -1
			if(top < -this._height)
				this._loop==0 ? this._stopScrolling() : this._loop==1 ? top = this._parent.getHeight() : top = parseInt(this._elScroller2.style.top) + this._height;
		}
		else if(this._direction == 'bottom')
		{
			top+= 1
			if(top > this._parent.getHeight())
				this._loop==0 ? this._stopScrolling() : this._loop==1 ? top = -this._height : top = parseInt(this._elScroller2.style.top) - this._height;
		}
		else if(this._direction == 'right')
		{
			left+= 1
			if(left > this._parent.getWidth())
				this._loop==0 ? this._stopScrolling() : this._loop==1 ? left= -this._width : left = parseInt(this._elScroller2.style.left) - this._width;
		}
		else if(this._direction == 'left')
		{
			left+= -1
			if(left < -this._width)
				this._loop==0 ? this._stopScrolling() : this._loop==1 ? left= this._parent.getWidth() : left = parseInt(this._elScroller2.style.left) + this._width;
		}
		this._elScroller.setStyle({top: top+'px', left: left+'px'});
		
		if(this._loop==2)
		{
			var top = parseInt(this._elScroller2.style.top) ;
			var left = parseInt(this._elScroller2.style.left) ;

			if(this._direction == 'top')
			{
				top+= -1
				if(top < -this._height)
					top = parseInt(this._elScroller.style.top) + this._height;
			}
			else if(this._direction == 'bottom')
			{
				top+= 1
				if(top > this._parent.getHeight())
					top = parseInt(this._elScroller.style.top) - this._height;
			}
			else if(this._direction == 'right')
			{
				left+= 1
				if(left > this._parent.getWidth())
					left = parseInt(this._elScroller.style.left) - this._width;
			}
			else if(this._direction == 'left')
			{
				left+= -1
				if(left < -this._width)
					left= parseInt(this._elScroller.style.left) + this._width;
			}
			this._elScroller2.setStyle({top: top+'px', left: left+'px'});
		}
	}
});