/**
 * validate.js
 *
 * @author 	Maxime Haineault <max@centdessin.com>
 * @version	0.8
 * @desc 	Library to handle forms and data validation
 * @todo
 *  - isValidEmail: fix 2 character domains validation
 */

Object.extend(Form.Methods,{	
  validate: function(element, action) {
  	hasErrors = false;
	$$('.validationErrors').each( function (el) {
		el.className = el.className.replace('validationErrors', '').trim();
	});
	var idSel = '#' + element.id + ' ';
	textareas = $$(idSel + 'textarea');
  	selects   = $$(idSel + 'select');
  	inputs    = $$(idSel + 'input').concat(textareas, selects);
  	inputs.each(function(input){
  		jsFF._LastElemValidatedBeforeError = input.id || input.name;
  		validations = input.className.match(/isValid\w+/g);
  		if (validations) validations.each(function(validation){
			var f = validation.match(/isValidCustom_(.*)[ ]*/);
			if (f && f.length > 1)
			{
				if (isFinite(input.value))
					var cVal = f[1] + '('  + input.value + ')';
				else
					var cVal = f[1] + "('"  + input.value.replace(/'/g,"\'") + "')";
				try {
					var res = eval(cVal) ;
				} catch(e) {
					jsFF.error('Error : ' + e.fileName + ' : line ' + e.lineNumber + ' => ' + e.message);
					jsFF.error('Context => class = ' + f[0] + ' - function : ' + f[1] + ' - value : ' + input.value);
					return false;
				}
				if (! res)
				{
					hasErrors = true;
					input.addClassName('validationErrors');
				}
			}
			else if (validation.match(/isValidEmpty.*/) && ! input[validation]())
			{
			}
			else if ( validation.match(/isValidSameAs_.*/))
			{
				var model = validation.replace(/isValidSameAs_(.*)/, "$1");
				if (model && $(model))
				{
					if (input.value != $F(model))
					{
						hasErrors = true;
						input.addClassName('validationErrors');
					}
				}
			}
			else
				if(!input[validation]()) 
				{
					hasErrors = true;
					if(input.className.match('isValidRadioRequired'))
					{
						var parent = input.parentNode ;
						while(parent && (parent.nodeName != 'DIV' && parent.nodeName != 'TR' && parent.nodeName != 'BODY'))
							parent = parent.parentNode ;
						if(parent && (parent.nodeName=='DIV' || parent.nodeName=='TR') )
							parent.className = parent.className + ' validationErrors';
						else
							jsFF.error('Error : validate.js : isValidRadioRequired without div container in dom.');
					}
					else
						input.addClassName('validationErrors');
				}
  		});
  	});
	jsFF._LastElemValidatedBeforeError = null;
  	if (hasErrors) {
		element.addClassName('formValidationErrors');
  		Form.Methods.validationErrorCallback(element);
  		return false;
  	}
  	else 
		if (action)
			return action(element);
		else
			return true;
  },
	validationErrorCallback: function(element) {
		element.getElementsByClassName('validationErrors')[0].focus();
	}
});

Object.extend(Form.Element.Methods,{
  setModified: function(element) {
  	$(element).addClassName('modified');
  },
  setUnModified: function(element) {
  	$(element).removeClassName('modified');
  },
  isModified: function(element) {
    return $(element).hasClassName('modified');
  },
  isRequired: function(element) {
    return $(element).hasClassName('isValidRequired');
  },
  isEmpty: function(element) {
  	return $F(element) == '';
  },
  isNotEmpty: function(element) {
  	return !$F(element) == '';
  },
  isValid: function(element) {
  	idValid = true;
  	element.className.match(/isValid\w{1,}/g).each(function(className) {
  		if(!$(element)[className]()) idValid = false;
  	});
  	return idValid;
  },
  isValidEmpty: function(element) {
  	return Form.Element.Methods.isEmpty(element);
  },
  isValidNotEmpty: function(element) {
  	return Form.Element.Methods.isNotEmpty(element);
  },
  isValidRequired: function(element) {
  	return Form.Element.Methods.isNotEmpty(element);
  },
  isValidBoolean: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	return !!$F(element).match(/^(0|1|true|false)$/);
  },
  isValidEmail: function(element) { // TODO: two letters domains like .ca or .us doesn't validate..
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	return !!$F(element).match(/^([a-zA-Z0-9]+([\.+_-][a-zA-Z0-9]+)*)@(([a-zA-Z0-9]+((\.|[-]{1,2})[a-zA-Z0-9]+)*)\.[a-zA-Z]{2,6})$/); 
  },	/*  /(^[a-z]([a-z_\.]*)@([a-z_\.]*)([.][a-z]{3})$)|(^[a-z]([a-z_\.]*)@([a-z_\-\.]*)(\.[a-z]{3})(\.[a-z]{2})*$)/i	*/
  isValidInteger: function(element) {
  	return !!$F(element).match(/(^-?\d+$)/);
  },
  isValidNumeric: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	return !!$F(element).match(/(^-?\d\d*[\.|,]\d*$)|(^-?\d\d*$)|(^-?[\.|,]\d\d*$)/);
  },
  isValidAplhaNumeric: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	return !!$F(element).match(/^[_\-a-z0-9]+$/gi);
  },
  // 0000-00-00 00:00:00 to 9999:12:31 59:59:59 (no it is not a "valid DATE" function)
  isValidDatetime: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
	switch(jsFF.locale)
	{
		case 'fr':
		case 'fr_UTF8':
			var dt = $F(element).match(/^(\d{2})-(\d{2})-(\d{4})\s(\d{2}):(\d{2}):(\d{2})$/);
			return dt && !!(dt[1]<=31 && dt[2]<=12 && dt[3]<=9999 && dt[4]<=59 && dt[5]<=59 && dt[6]<=59) || false;
			break;
		case 'en' :
			var dt = $F(element).match(/^(\d{2})-(\d{2})-(\d{4})\s(\d{2}):(\d{2}):(\d{2})$/);
			return dt && !!(dt[1]<=12 && dt[2]<=31 && dt[3]<=9999 && dt[4]<=59 && dt[5]<=59 && dt[6]<=59) || false;
			break;
		case 'default':
			var dt = $F(element).match(/^(\d{4})-(\d{2})-(\d{2})\s(\d{2}):(\d{2}):(\d{2})$/);
			break;
	}
  	return dt && !!(dt[1]<=9999 && dt[2]<=12 && dt[3]<=31 && dt[4]<=59 && dt[5]<=59 && dt[6]<=59) || false;
  },
  // 0000-00-00 to 9999-12-31
  isValidDate: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
	switch(jsFF.locale)
	{
		case 'fr':
		case 'fr_UTF8':
			var d = $F(element).match(/^(\d{2})[-|\/](\d{2})[-|\/](\d{4})$/);
			return d && !!(d[1]<=31 && d[2]<=12 && d[3]<=9999) || false;
			break;
		case 'en' :
			var d = $F(element).match(/^(\d{2})[-|\/](\d{2})[-|\/](\d{4})$/);
			return d && !!(d[2]<=12 && d[1]<=31 &&d[3]<=9999) || false;
			break;
		case 'default':
			var d = $F(element).match(/^(\d{4})[-|\/](\d{2})[-|\/](\d{2})$/);
			break;
	}
		return d && !!(d[1]<=9999 && d[2]<=12 && d[3]<=31) || false;
  },
  // 00:00:00 to 59:59:59
  isValidTime: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	var t = $F(element).match(/^(\d{1,2}):(\d{1,2}):(\d{1,2})$/);
  	return t && !!(t[1]<=24 && t[2]<=59 && t[3]<=59) || false;
  },
  // 0.0.0.0 to 255.255.255.255
  isValidIPv4: function(element) { 
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	ip = $F(element).match(/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/);
  	return ip && !!(ip[1]<=255 && ip[2]<=255 && ip[3]<=255 && ip[4]<=255) || false;
  },
  isValidCurrency: function(element) { // Q: Should I consider those signs valid too ? : ¢|€|₤|₦|¥
  	return $F(element).match(/^\$?\s?\d+?[\.,\,]?\d+?\s?\$?$/) && true || false;
  },
  isValidSelectRequired: function(element) {
	var val = $F(element);
	return (val != '') && true || false;
  },
  // Social Security Number (999-99-9999 or 999999999)
  isValidSSN: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	return $F(element).match(/^\d{3}\-?\d{2}\-?\d{4}$/) && true || false;
  },
  // Social Insurance Number (999999999)
  isValidSIN: function(element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
  	return $F(element).match(/^\d{9}$/) && true || false;
  },
  isValidRadioRequired: function(element) {
	var res = false;
	var tab = Form.getInputs(element.form, 'radio', element.name);
	for(var i = 0; i < tab.length ; i++)
		if(tab[i].checked){
			res=true; break;
		}
	return res;
  },
  isValidRegexp: function (element) {
	if (element.className.match(/isValidEmpty/) && Form.Element.Methods.isEmpty(element))
		return true;
	if ( element.id  && element.id != '')
		if (eval("typeof (Expr_" + element.id + ");") == 'object')
			return eval ("Expr_" + element.id + ".test($F(element))");
		else
			jsFF.error('Error : an isValidRegExp validator was defined but there is not an existing Regexp object named Expr_' + element.id);
	else
		jsFF.error('Error : an isValidRegExp validator was defined but the element don\'t have an id.');
  }
});
Element.addMethods();
$V = function(el) { return $(el).isValid(); }
